package org.jeecg.modules.contract.file.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.contract.file.entity.ContractFile;
import org.jeecg.modules.contract.file.service.IContractFileService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 合同档案
 * @author： jeecg-boot
 * @date：   2021-11-17
 * @version： V1.0
 */
@RestController
@RequestMapping("/contract.file/contractFile")
@Slf4j
public class ContractFileController {
	@Autowired
	private IContractFileService contractFileService;
	
	/**
	  * 分页列表查询
	 * @param contractFile
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<ContractFile>> queryPageList(ContractFile contractFile,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<ContractFile>> result = new Result<IPage<ContractFile>>();
		QueryWrapper<ContractFile> queryWrapper = QueryGenerator.initQueryWrapper(contractFile, req.getParameterMap());
		queryWrapper.lambda().eq(ContractFile::getDelFlag,"0");
		queryWrapper.lambda().orderByDesc(ContractFile::getCreateTime);
		Page<ContractFile> page = new Page<ContractFile>(pageNo, pageSize);
		IPage<ContractFile> pageList = contractFileService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param contractFile
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<ContractFile> add(@RequestBody ContractFile contractFile) {
		Result<ContractFile> result = new Result<ContractFile>();
		try {
			contractFileService.save(contractFile);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 主表和子表数据一起保存
	* @Date 2021/11/17 16:41
	* @Param [contractFile]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.contract.file.entity.ContractFile>
	*/
	 @PostMapping(value = "/saveMainAndChild")
	 public Result<ContractFile> saveMainAndChild(@RequestBody ContractFile contractFile) {
		 Result<ContractFile> result = new Result<ContractFile>();
		 try {
			 contractFileService.saveMainAndChild(contractFile);
			 result.success("添加成功！");
		 } catch (Exception e) {
			 e.printStackTrace();
			 log.info(e.getMessage());
			 result.error500("操作失败");
		 }
		 return result;
	 }
	
	/**
	  *  编辑
	 * @param contractFile
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<ContractFile> edit(@RequestBody ContractFile contractFile) {
		Result<ContractFile> result = new Result<ContractFile>();
		ContractFile contractFileEntity = contractFileService.getById(contractFile.getId());
		if(contractFileEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = contractFileService.updateById(contractFile);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 主表和子表一起修改
	* @Date 2021/11/17 17:19
	* @Param [contractFile]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.contract.file.entity.ContractFile>
	*/
	 @PutMapping(value = "/updateMainAndChild")
	 public Result<ContractFile> updateMainAndChild(@RequestBody ContractFile contractFile) {
		 Result<ContractFile> result = new Result<ContractFile>();
		 ContractFile contractFileEntity = contractFileService.getById(contractFile.getId());
		 if(contractFileEntity==null) {
			 result.error500("未找到对应实体");
		 }else {
			 try {
				 contractFileService.updateMainAndChild(contractFile);
				 result.success("修改成功");
			 } catch (Exception e) {
				 e.printStackTrace();
				 result.error500("修改失败："+e.getMessage());
			 }
		 }
		 return result;
	 }
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<ContractFile> delete(@RequestParam(name="id",required=true) String id) {
		Result<ContractFile> result = new Result<ContractFile>();
		ContractFile contractFile = contractFileService.getById(id);
		if(contractFile==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = contractFileService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 主表与子表一起删除
	* @Date 2021/11/17 17:29
	* @Param [id]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.contract.file.entity.ContractFile>
	*/
	 @DeleteMapping(value = "/deleteMainAndChild")
	 public Result<ContractFile> deleteMainAndChild(@RequestParam(name="id",required=true) String id) {
		 Result<ContractFile> result = new Result<ContractFile>();
		 ContractFile contractFile = contractFileService.getById(id);
		 if(contractFile==null) {
			 result.error500("未找到对应实体");
		 }else {
			 try {
				 contractFileService.deleteMainAndChild(contractFile);
				 result.success("删除成功");
			 } catch (Exception e) {
				 e.printStackTrace();
				 result.error500("删除失败："+e.getMessage());
			 }
		 }
		 return result;
	 }
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<ContractFile> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<ContractFile> result = new Result<ContractFile>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.contractFileService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<ContractFile> queryById(@RequestParam(name="id",required=true) String id) {
		Result<ContractFile> result = new Result<ContractFile>();
		ContractFile contractFile = contractFileService.getById(id);
		if(contractFile==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(contractFile);
			result.setSuccess(true);
		}
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 根据主表id查询主子表数据
	* @Date 2021/11/17 17:31
	* @Param [id]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.contract.file.entity.ContractFile>
	*/
	 @GetMapping(value = "/queryMainAndChildById")
	 public Result<ContractFile> queryMainAndChildById(@RequestParam(name="id",required=true) String id) {
		 Result<ContractFile> result = new Result<ContractFile>();
		 ContractFile contractFile = contractFileService.getById(id);
		 if(contractFile==null) {
			 result.error500("未找到对应实体");
		 }else {
		 	//获取所有子表数据
			 contractFileService.queryMainAndChildById(contractFile);
			 result.setResult(contractFile);
			 result.setSuccess(true);
		 }
		 return result;
	 }

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<ContractFile> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              ContractFile contractFile = JSON.parseObject(deString, ContractFile.class);
              queryWrapper = QueryGenerator.initQueryWrapper(contractFile, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<ContractFile> pageList = contractFileService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "合同档案列表");
      mv.addObject(NormalExcelConstants.CLASS, ContractFile.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("合同档案列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<ContractFile> listContractFiles = ExcelImportUtil.importExcel(file.getInputStream(), ContractFile.class, params);
              for (ContractFile contractFileExcel : listContractFiles) {
                  contractFileService.save(contractFileExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listContractFiles.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
